﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Threading;
using System.Windows;
using System.Windows.Forms;
using System.Windows.Threading;
using System.Windows.Media.Imaging;
using System.Windows.Media;

namespace CreationTimeChanger
{
    /// <summary>
    /// Interaktionslogik für MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        private string path = string.Empty;
        private string[] files = new string[0];

        public MainWindow()
        {
            InitializeComponent();
        }

        private void PfadButton_Click(object sender, RoutedEventArgs e)
        {
            OpenFolderDialog();
        }

        private void PfadTextBox_MouseDoubleClick(object sender, System.Windows.Input.MouseButtonEventArgs e)
        {
            OpenFolderDialog();
        }

        private void RekursivCheckBox_Click(object sender, RoutedEventArgs e)
        {
            if (!path.Equals(string.Empty))
                ReadDirectory();
        }

        public void ReadDirectory()
        {
            try
            {
                if (RekursivCheckBox.IsChecked == true)
                    files = Directory.GetFiles(path, "*", SearchOption.AllDirectories);
                else
                    files = Directory.GetFiles(path, "*", SearchOption.TopDirectoryOnly);
            }
            catch (Exception ex)
            {
                System.Windows.MessageBox.Show(ex.Message);
            }

            ZahlLabel.Content = files.Length + " Dateien gefunden";

            if (files.Length > 0)
                WorkButtonImage.Source = new BitmapImage(new Uri("/CreationTimeChanger;component/112_RightArrowShort_Green_48x48_72.png", UriKind.Relative));
            else
                WorkButtonImage.Source = new BitmapImage(new Uri("/CreationTimeChanger;component/112_RightArrowShort_Orange_48x48_72.png", UriKind.Relative));
        }

        private void OpenFolderDialog()
        {
            FolderBrowserDialog dialog = new FolderBrowserDialog();
            if (!path.Equals(string.Empty))
                dialog.SelectedPath = path;
            dialog.ShowNewFolderButton = false;

            DialogResult result = dialog.ShowDialog();
            if (result == System.Windows.Forms.DialogResult.OK)
            {
                path = dialog.SelectedPath;
                PfadTextBox.Text = path;
                ReadDirectory();
            }
        }

        private void WorkButton_Click(object sender, RoutedEventArgs e)
        {
            if (files.Length == 0)
            {
                System.Windows.MessageBox.Show("Keine Dateien gefunden.");
                return;
            }

            Fortschrittsbalken.Value = 0;
            Fortschrittsbalken.Maximum = files.Length - 1;
            Fortschrittsbalken.Foreground = Brushes.Red;


            Thread processFilesThread = new Thread(new ThreadStart(this.ProcessFiles));
            processFilesThread.Start();
        }

        private void ShowErrorLog(List<string> ErrorList)
        {
            new ErrorWindow(ErrorList).Show();
        }

        private void ProcessFiles()
        {
            string incompleteLabelText = "/" + files.Length + " Dateien verarbeitet";
            List<string> errorList = new List<string>();

            try
            {
                for (int i = 0; i < files.Length; i++)
                {
                    string file = files[i];

                    int fileYear = 0;
                    int fileMonth = 0;
                    int fileDay = 0;
                    int fileHour = 0;
                    int fileMinute = 0;
                    int fileSecond = 0;

                    int fileNamePosition = file.LastIndexOf('\\') + 1;
                    int.TryParse(file.Substring(fileNamePosition, 4), out fileYear);
                    int.TryParse(file.Substring(fileNamePosition + 5, 2), out fileMonth);
                    int.TryParse(file.Substring(fileNamePosition + 8, 2), out fileDay);
                    int.TryParse(file.Substring(fileNamePosition + 11, 2), out fileHour);
                    int.TryParse(file.Substring(fileNamePosition + 14, 2), out fileMinute);
                    int.TryParse(file.Substring(fileNamePosition + 17, 2), out fileSecond);

                    if (fileYear == 0 || fileMonth == 0 || fileDay == 0 || fileHour > 23 || fileMinute > 59 || fileSecond > 59)
                    {
                        errorList.Add(file);
                        continue;
                    }

                    DateTime newTime = new DateTime(fileYear, fileMonth, fileDay, fileHour, fileMinute, fileSecond);

                    File.SetCreationTime(file, newTime);

                    this.Dispatcher.Invoke(DispatcherPriority.Normal, new Action(() => { this.ZahlLabel.Content = i + incompleteLabelText; }));
                    this.Dispatcher.Invoke(DispatcherPriority.Normal, new Action(() => { this.Fortschrittsbalken.Value = i; }));
                }
            }
            catch (Exception ex)
            {
                System.Windows.MessageBox.Show(ex.Message + "\n\n" + ex.StackTrace);
            }

            this.Dispatcher.Invoke(DispatcherPriority.Normal, new Action(() => { this.Fortschrittsbalken.Foreground = Brushes.Green; }));
            this.Dispatcher.Invoke(DispatcherPriority.Normal, new Action(() => { this.ZahlLabel.Content = files.Length + " Dateien verarbeitet"; }));
            this.Dispatcher.Invoke(DispatcherPriority.Normal, new Action(() => { this.ShowErrorLog(errorList); }));
            Thread.Sleep(1000);

            this.Dispatcher.Invoke(DispatcherPriority.Normal, new Action(() => { this.Fortschrittsbalken.Foreground = Brushes.Red; }));
            this.Dispatcher.Invoke(DispatcherPriority.Normal, new Action(() => { this.Fortschrittsbalken.Value = 0; }));
            this.Dispatcher.Invoke(DispatcherPriority.Normal, new Action(() => { this.ZahlLabel.Content = string.Empty; }));
        }
    }
}
